/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { Legend } from "../../types/topology/Legend";
import { SGraphItem } from "@persagy-web/graph/lib";
import { SLineItem } from "../SLineItem";
import { SPoint } from "@persagy-web/draw/lib";

/**
 * 图例节点Item(线类型)
 *
 * * @author  张宇(taohuzy@163.com)
 */
export class SLineLegendItem extends SLineItem {
    /** 图例节点对象数据 */
    data: Legend;

    /** x轴缩放属性 */
    _scaleX: number = 1;
    get scaleX(): number {
        return this._scaleX;
    }
    set scaleX(v: number) {
        this._scaleX = v;
        if (this.data.Scale) {
            this.data.Scale.X = v;
        }
        this.update();
    }

    /** y轴缩放属性 */
    _scaleY: number = 1;
    get scaleY(): number {
        return this._scaleY;
    }
    set scaleY(v: number) {
        this._scaleY = v;
        if (this.data.Scale) {
            this.data.Scale.Y = v;
        }
        this.update();
    }

    /** y轴旋转属性 */
    _rolate: number = 0;
    get rolate(): number {
        return this._rolate;
    }
    set rolate(v: number) {
        this._rolate = v;
        if (this.data.Rolate) {
            this.data.Rolate.Z = v;
        }
        this.update();
    }

    set name(v: string) {
        this.data.Name = v;
    }

    set line(arr: SPoint[]) {
        if (this.data.Properties) {
            this.data.Properties.Line = arr;
        }
    }

    set x(v: number) {
        this.data.Pos.X = v;
    }

    set y(v: number) {
        this.data.Pos.Y = v;
    }

    set width(v: number) {
        if (this.data.Size) {
            this.data.Size.Width = v;
        }
    }

    set height(v: number) {
        if (this.data.Size) {
            this.data.Size.Height = v;
        }
    }

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      图例节点对象数据
     */
    constructor(parent: SGraphItem | null, data: Legend) {
        super(parent);
        this.data = data;
        this.moveTo(data.Pos.X, data.Pos.Y);
        if (this.data.Scale) {
            this.scaleX = this.data.Scale.X;
            this.scaleY = this.data.Scale.Y;
        }
        if (this.data.Rolate && this.data.Rolate.Z) {
            this.rolate = this.data.Rolate.Z;
        }
        if (this.data.Size) {
            this.width = this.data.Size.Width;
            this.height = this.data.Size.Height;
        }
        if (this.data.Properties && this.data.Properties.Line) {
            this.line = this.data.Properties.Line;
        }
    }
} // Class SLineLegendItem
