/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SColor, SGradientStop, SPoint } from "./";

/**
 * 渐变颜色节点
 *
 * @author  庞利祥(sybotan@126.com)
 */
export abstract class SGradient {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** 起点 */
    start = new SPoint();

    /** 结束点 */
    end = new SPoint();

    /** 起点X坐标 */
    get x1(): number {
        return this.start.x;
    } // Get x1
    set x1(value: number) {
        this.start.x = value;
    } // Set x1

    /** 起点Y坐标 */
    get y1(): number {
        return this.start.y;
    } // Get y1
    set y1(value: number) {
        this.start.y = value;
    } // Set y1

    /** 终点X坐标 */
    get x2(): number {
        return this.end.x;
    } // Get x2
    set x2(value: number) {
        this.end.x = value;
    } // Set x2

    /** 终点Y坐标 */
    get y2(): number {
        return this.end.y;
    } // Get y2
    set y2(value: number) {
        this.end.y = value;
    } // Set y2

    /** 渐变颜色节点列表 */
    stopList: SGradientStop[] = [];

    /**
     * 添加渐变颜色节点
     *
     * @param   pos     位置[0-1]
     * @param   color   颜色
     */
    addColorStop(pos: number, color: SColor): void {
        this.stopList.push(new SGradientStop(pos, color));
    } // Function addColorStop()

    /**
     * 设置起点坐标
     *
     * @param   x       X坐标
     * @param   y       Y坐标
     */
    setStart(x: number, y: number): void {
        this.start.x = x;
        this.start.y = y;
    } // Function setStart()

    /**
     * 设置终点坐标
     *
     * @param   x       X坐标
     * @param   y       Y坐标
     */
    setEnd(x: number, y: number): void {
        this.end.x = x;
        this.end.y = y;
    } // Function setEnd()
} // Class SGradientStop
